@@ -22,22 +22,19 @@ pub type Inc<T, U> = <U as AddLength<T, U1>>::Output;
 #[macro_export]
 macro_rules! arr_impl {
     ($T:ty; $N:ty, [$($x:expr),*], []) => ({
-        use generic_array::typenum::U0;
-        use generic_array::GenericArray;
-        use generic_array::arr::Inc;
-        GenericArray::<$T, $N>::clone_from_slice(&[$($x),*])
+	unsafe { $crate::transmute::<_, $crate::GenericArray<$T, $N>>([$($x),*]) }
     });
     ($T:ty; $N:ty, [], [$x1:expr]) => (
-        arr_impl!($T; Inc<$T, $N>, [$x1], [])
+        arr_impl!($T; $crate::arr::Inc<$T, $N>, [$x1 as $T], [])
     );
     ($T:ty; $N:ty, [], [$x1:expr, $($x:expr),+]) => (
-        arr_impl!($T; Inc<$T, $N>, [$x1], [$($x),*])
+        arr_impl!($T; $crate::arr::Inc<$T, $N>, [$x1 as $T], [$($x),*])
     );
     ($T:ty; $N:ty, [$($y:expr),+], [$x1:expr]) => (
-        arr_impl!($T; Inc<$T, $N>, [$($y),*, $x1], [])
+        arr_impl!($T; $crate::arr::Inc<$T, $N>, [$($y),*, $x1 as $T], [])
     );
     ($T:ty; $N:ty, [$($y:expr),+], [$x1:expr, $($x:expr),+]) => (
-        arr_impl!($T; Inc<$T, $N>, [$($y),*, $x1], [$($x),*])
+        arr_impl!($T; $crate::arr::Inc<$T, $N>, [$($y),*, $x1 as $T], [$($x),*])
     );
 }
 
@@ -45,8 +42,11 @@ macro_rules! arr_impl {
 /// Example: `let test = arr![u32; 1, 2, 3];`
 #[macro_export]
 macro_rules! arr {
+    ($T:ty;) => ({
+        unsafe { $crate::transmute::<[$T; 0], $crate::GenericArray<$T, $crate::typenum::U0>>([]) }
+    });
     ($T:ty; $($x:expr),*) => (
-        arr_impl!($T; U0, [], [$($x),*])
+        arr_impl!($T; $crate::typenum::U0, [], [$($x),*])
     );
     ($($x:expr,)+) => (arr![$($x),*]);
     () => ("""Macro requires a type, e.g. `let array = arr![u32; 1, 2, 3];`")
